#region usings
using System;
using System.ComponentModel.Composition;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;

using System.Threading;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "EvalOrder", Category = "Test", Help = "Basic template with one value in/out", Tags = "c#")]
	#endregion PluginInfo
	public class TestEvalOrderNode : IPluginEvaluate, IPartImportsSatisfiedNotification, IDisposable
	{
		#region fields & pins
		[Input("Input")]
		public ISpread<double> FInput;
		
		[Input("Do Log")]
		public ISpread<bool> FDoLog;

		[Output("Output")]
		public ISpread<double> FOutput;

		[Output("Node Path")]
		public ISpread<string> FPath;
		
		[Import()]
		public ILogger FLogger;
		
		[Import]
		public IMainLoop FMainloop;
		
		[Import]
		public IPluginHost2 FHost;
		#endregion fields & pins
		
		public void OnImportsSatisfied()
		{
			FMainloop.OnPrepareGraph += OnPrepareGraph;
			FPath[0] = FHost.GetNodePath(false);
		}
		
		public void Dispose()
		{
			FMainloop.OnPrepareGraph -= OnPrepareGraph;
		}
		
		public void OnPrepareGraph(object sender, EventArgs e)
		{
			if (FDoLog[0])
				FLogger.Log(LogType.Debug, CreateMessage("Prepare Graph"));
		}

		string CreateMessage(string message)
		{
			var t = DateTime.Now;
			var msg = t.ToLongTimeString()+":"+t.Millisecond+"."+t.Ticks;
			msg += " "+FHost.GetNodePath(false);
			msg += " "+message;
			return msg;
		}
		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			if (FDoLog[0])
				FLogger.Log(LogType.Debug, CreateMessage("Evaluate"));
			
			Thread.Sleep(10);
			
		}
	}
}
